' BLITZMAX STD-LIB
' ~~   OBJECT   ~~


Rem
bbdoc: StdObject is the root class of the most Std.Mod classes.<br>Through StdObject, objects inherits a basic interface To the reflection methods.
EndRem
Type StdObject Abstract
	
	Field __typeid__:TTypeId

	'----
	'init
	'----

	Rem
	bbdoc: awakes the receiver.
	EndRem
	Method awake()
		'awake code here...
	EndMethod
	
	'----------
	'reflection
	'----------
	
	Rem
	bbdoc: returns the recievers <i>typeid</i>.
	EndRem
	Method typeID:TTypeId()
		If Not __typeid__ Then __typeid__ = TTypeId.ForObject( Self )
		Return __typeid__
	EndMethod
	
	Rem
	about returns the recievers Super Type <i>typeid</i>.
	EndRem
	Method superTypeID:TTypeId()
		Return typeID().superType()
	EndMethod
	
	Rem
	bbdoc: returns the recievers Type name.
	EndRem
	Method typeName:String()
		Return typeID().name()
	EndMethod
	
	Rem
	bbdoc: returns all fields of the recievers <i>Type</i>.
	EndRem
	Method fields:TList()
		Return typeID().fields()
	EndMethod
	
	Rem
	bbdoc: returns all methods of the recievers <i>Type</i>.
	EndRem
	Method methods:TList()
		Return typeID().methods()
	EndMethod
	
	Rem
	bbdoc: returns all derived types of the recievers <i>Type</i>.
	EndRem
	Method derivedTypes:TList()
		Return typeID().derivedTypes()
	EndMethod
	
	Rem
	bbdoc: returns the meta data of the recievers Type, For the given key.
	EndRem
	Method metaDataForKey:String( key_:String="" )
		Return typeID().metaData( key_ )
	EndMethod
	
	Rem
	bbdoc: returns If the recievers Type Extends the given <i>typeid</i>.
	EndRem
	Method extendsType:Int( type_:TTypeId )
		If type_ = Null 
			Throw String("invalid typeid passed to " + typeName())
			Return 0
		Else
			Return typeID().extendsType( type_ )
		EndIf
	EndMethod
	
	Rem
	bbdoc: returns the recievers Type elemt Type.
	EndRem
	Method elementType:TTypeId()
		Return typeID().elementType()
	EndMethod
	
	Rem
	bbdoc: performs the given Method on the reciever.<br>returns the Method returned <i>Object</i>. 
	EndRem
	Method invokeMethod:Object( method_:TMethod, data_:Object[] )
		If method_ = Null 
			Throw String("invalid method passed to " + typeName())
			Return Null
		Else
			Return method_.invoke( Self, data_ )
		EndIf
	EndMethod
	
	Rem
	bbdoc: returns If the reciever responds To the given Method.
	EndRem
	Method respondsToMethod:Int( method_:String )
		If typeID().findMethod( method_ )
			Return YES
		Else
			Return NO
		EndIf
	EndMethod
	
	Rem
	bbdoc: reuturns the recievers Method For the given name.
	EndRem
	Method methodForName:TMethod( name_:String )
		Return typeID().findMethod( name_ )
	EndMethod
	
	Rem
	bbdoc: reuturns the recievers Field For the given name.
	EndRem
	Method fieldForName:TField( name_:String )
		Return typeID().findField( name_ )
	EndMethod

	Method stopPoseAsType()
		__typeid__ = Null
		typeID()
	EndMethod
	
	'-------
	'set/get
	'-------
	
	Rem
	bbdoc: sets the recievers <i>Int</i> value.
	EndRem
	Method setIntValue( int_:Int )
		'...
	EndMethod
	
	Rem
	bbdoc: returns the recievers <i>Int</i> value.
	EndRem
	Method intValue:Int()
		Return 0
	EndMethod
	
	Rem
	bbdoc: sets the recievers <i>Float</i> value.
	EndRem
	Method setFloatValue( float_:Float )
		'...
	EndMethod
	
	Rem
	bbdoc: returns the recievers <i>Float</i> value.
	EndRem
	Method floatValue:Float()
		Return 0.0
	EndMethod
	
	Rem
	bbdoc: sets the recievers <i>Double</i> value.
	EndRem
	Method setDoubleValue( double_:Double )
		'...
	EndMethod
	
	Rem
	bbdoc: returns the recievers <i>Double</i> value.
	EndRem
	Method doubleValue:Double()
		Return 0.0
	EndMethod
	
	Rem
	bbdoc: sets the recievers <i>String</i> value.
	EndRem
	Method setStringValue( string_:String )
		'...
	EndMethod
	
	Rem
	bbdoc: returns the recievers <i>String</i> value.
	EndRem
	Method stringValue:String()
		Return ""
	EndMethod
	
	'--------
	'byte ptr
	'--------
	
	Rem
	bbdoc: returns the <i>Byte</i> pointer of the reciever.
	EndRem
	Method bytePtr:Byte Ptr()
		Return Byte Ptr( Self ) 
	EndMethod
	
	'----
	'hash
	'----
	
	Rem
	bbdoc: returns the hash value of the reciever.
	EndRem
	Method hash:Int()
		Throw Self.typeName() + " does not implement 'hash'"
	EndMethod
	
	'-------
	'compare
	'-------
	
	Rem
	bbdoc: returns YES If the given Object is equal To the reciever instance..
	EndRem
	Method isEqual:Int( object_:StdObject )
		Return Self = object_
	EndMethod
	
	Method compare:Int( object_:Object )
		Return Super.Compare(object_)
	EndMethod
	
	'----
	'copy
	'----
	
	Rem
	bbdoc: returns a deep copy of the reciever.
	EndRem
	Method copy:StdObject()
		Throw Self.typeName() + " does not implement 'copy'"
	EndMethod
	
	'-----------
	'description
	'-----------
	
	Rem
	bbdoc: returns the description of the reciever.
	about: @{toString} returns @{description}, too.
	EndRem
	Method description:String()
		Return typeName()
	EndMethod
	
	Method toString:String()
		Return Self.description()
	EndMethod
	
	Rem
	bbdoc: returns the description of the recievers class.
	EndRem
	Method classDescription:String()
		Return typeName()
	EndMethod
	
End Type

Rem
bbdoc: TypeId of the #StdObject class.
End Rem
Global StdObjectTypeId:TTypeId = TTypeId.ForName("StdObject")
